/***************************************************************************
*   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#include "face.h"

#include <cmath>

namespace Papyrus {

Face::Face( double size, TYPE type ) :
    m_type( type ),
    m_size( size )
    {
      this->set_extents( Region( -m_size, -m_size, 2.0 * m_size, 2.0 * m_size ) );
    }

    Face::pointer Face::create( double size, TYPE type ) {
        return Face::pointer ( new Face( size, type ) );
    }

    Face::~Face() { }

    void Face::draw( Cairo::RefPtr<Cairo::Context> cairo ) {
        cairo->arc( 0.0, 0.0, m_size, 0.0, 2*M_PI);
        cairo->set_source( Cairo::SolidPattern::create_rgba(1.0, 1.0, 0.0, 1.0));
        cairo->fill( );

        cairo->arc( -m_size/3.0, -m_size/3.0, m_size/6.0, 0.0, 2*M_PI);
        cairo->set_source( Cairo::SolidPattern::create_rgba(0.0, 0.0, 0.0, 1.0));
        cairo->fill( );

        cairo->arc( m_size/3.0, -m_size/3.0, m_size/6.0, 0.0, 2*M_PI);
        cairo->set_source( Cairo::SolidPattern::create_rgba(0.0, 0.0, 0.0, 1.0));
        cairo->fill( );

        cairo->set_line_width(m_size/6.0);
        cairo->set_line_cap(Cairo::LINE_CAP_ROUND);
        cairo->move_to( -m_size/2.0, m_size/3.0);
        cairo->curve_to(-m_size/4.0, m_size/3.0*2.0, m_size/4.0, m_size/3.0*2.0, m_size/2.0, m_size/3.0);
        cairo->set_source( Cairo::SolidPattern::create_rgba(0.0, 0.0, 0.0, 1.0));
        cairo->stroke();
    }

    Face::TYPE Face::type() {
        return m_type;
    }

    void Face::set_type( TYPE type ) {
        m_type = type;
        this->need_redraw();
    }

    double Face::size() {
        return m_size;
    }

    void Face::set_size( double size ) {
        m_size = size;
        this->set_extents( Region( -m_size, -m_size, 2.0 * m_size, 2.0 * m_size ) );
        this->need_redraw();
    }

    void Face::set_size_type( double size, TYPE type ) {
        m_size = size;
        m_type = type;
        this->set_extents( Region( -m_size, -m_size, 2.0 * m_size, 2.0 * m_size ) );
        this->need_redraw();
    }

}
