/***************************************************************************
*   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#include "kochcurve.h"

namespace Papyrus
{

KochCurve::KochCurve(unsigned level, double size):
m_level(level), m_size(size)
  {
    this->create_vertices();
  }


KochCurve::pointer KochCurve::create( unsigned level, double size )
{
  return KochCurve::pointer( new KochCurve( level, size ) );
}

KochCurve::~KochCurve()
  {}

unsigned KochCurve::level( )
{
  return m_level;
}

void KochCurve::set_level( unsigned level )
{
  m_level = level;
  this->create_vertices();
}

double KochCurve::size( )
{
  return m_size;
}

void KochCurve::set_size( double size )
{
  m_size = size;
  this->create_vertices();
}

void KochCurve::create_vertices( )
{
  m_vertices.clear();

  m_lsize = m_size;
  for ( unsigned i=0; i < m_level; i++ )
    m_lsize /= 3.0;

  m_angle = 0.0;

  m_centroid_x = m_size/2.0;
  m_centroid_y = m_size*sin(M_PI/3.0)/6.0;

  m_vertices.push_back( Vertex(0.0, m_size*sin(M_PI/3.0)/3.0) );

  this->kochcurve( m_level );
  m_vertices.push_back( Vertex(m_lsize*cos(m_angle), m_lsize*sin(m_angle), true) );

  this->on_vertices_changed();
}

void KochCurve::kochcurve( unsigned level )
{
  static const double pi3 = M_PI / 3.0;

  if ( level > 0 )
  {
    this->kochcurve( level-1 );
    m_vertices.push_back( Vertex(m_lsize*cos(m_angle), m_lsize*sin(m_angle), true) );
    m_angle -= pi3;
    this->kochcurve( level-1 );
    m_vertices.push_back( Vertex(m_lsize*cos(m_angle), m_lsize*sin(m_angle), true) );
    m_angle += pi3 + pi3;
    this->kochcurve( level-1 );
    m_vertices.push_back( Vertex(m_lsize*cos(m_angle), m_lsize*sin(m_angle), true) );
    m_angle -= pi3;
    this->kochcurve( level-1 );
  }
}

}
