/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "marker.h"

namespace Papyrus {

    Marker::Marker(STYLE style, double width, double height, FACING facing)
: Shape(),
    m_style(style),
    m_width(width),
    m_height(height),
    m_facing(facing) {
			m_outline.set_join( Cairo::LINE_JOIN_MITER );
    }

    Marker::pointer Marker::create(STYLE style, double width, double height, FACING facing) {
        return Marker::pointer( new Marker( style, width, height, facing ) );
    }


    Marker::~Marker() {}

    Marker::STYLE Marker::style() const {
        return m_style;
    }

    void Marker::set_style(STYLE style) {
            m_style = style;
            this->shape_changed();
            m_signal_style.emit();
    }

    double Marker::width() const {
        return m_width;
    }

    void Marker::set_width(double width) {
            m_width = width;
            this->shape_changed();
            m_signal_size.emit();
    }

    double Marker::height() const {
        return m_height;
    }

    void Marker::set_height(double height) {
            m_height = height;
            this->shape_changed();
            m_signal_size.emit();
    }

    void Marker::width_height(double& width, double& height) const {
        width = m_width;
        height = m_height;
    }

    void Marker::set_width_height(double width, double height) {
      m_width = width;
      m_height = height;
      this->shape_changed();
      m_signal_size.emit();
    }

    Marker::FACING Marker::facing() const {
      return m_facing;
    }

    void Marker::set_facing(FACING facing) {
        m_facing = facing;
        this->shape_changed();
        m_signal_facing.emit();
    }

LineStyle& Marker::line_style() {
  return m_outline;
}


    void Marker::draw_shape( Cairo::RefPtr<Cairo::Context> cairo) {

      double dirmod = 1.0;
      if (m_facing == LEFT)
        dirmod = -1.0;

      switch (m_style) {
        case NONE:
            break;
        case LINES:
          cairo->move_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(0.0, 0.0);
          cairo->line_to(dirmod * -m_width, m_height/2.0);
            break;
        case TRIANGLE:
          cairo->move_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(0.0, 0.0);
          cairo->line_to(dirmod * -m_width, m_height/2.0);
          cairo->line_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(0.0, 0.0);
          break;
        case DIAMOND:
          cairo->move_to(dirmod * -m_width/2.0, -m_height/2.0);
          cairo->line_to(0.0, 0.0);
          cairo->line_to(dirmod * -m_width/2.0, m_height/2.0);
          cairo->line_to(dirmod * -m_width, 0.0);
          cairo->line_to(dirmod * -m_width/2.0, -m_height/2.0);
          cairo->line_to(0.0, 0.0);
          break;
        case HALF_DIAMOND:
          cairo->move_to(dirmod * -m_width/2.0, -m_height/2.0);
          cairo->line_to(dirmod * -m_width, 0.0);
          cairo->line_to(dirmod * -m_width/2.0, m_height/2.0);
          break;
        case HALF_HEAD_UPPER:
          cairo->move_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(0.0, 0.0);
          break;
        case HALF_HEAD_LOWER:
          cairo->move_to(dirmod * -m_width, m_height/2.0);
          cairo->line_to(0.0, 0.0);
          break;
        case SLASHED_CROSS:
          cairo->move_to(0.0, -m_height/2.0);
          cairo->line_to(dirmod * -m_width, m_height/2.0);

          cairo->move_to(dirmod * -m_width/2.0, -m_height/2.0);
          cairo->line_to(dirmod * -m_width/2.0, m_height/2.0);
            break;
        case ELLIPSE:
          cairo->arc(dirmod * -m_width/2.0, 0.0, m_width/2.0, 0.0, 2.0*M_PI);
            break;
        case DOT:
          cairo->arc(dirmod * -m_width/4.0, 0.0, m_width/4.0, 0.0, 2.0*M_PI);
          cairo->move_to(dirmod * -m_width/4.0, -m_width/2.0);
          cairo->line_to(dirmod * -m_width/4.0, m_width/2.0);
          break;
        case DOUBLE_TRIANGLE:
          cairo->move_to(dirmod * -m_width/2.0, -m_height/2.0);
          cairo->line_to(0.0, 0.0);
          cairo->line_to(dirmod * -m_width/2.0, m_height/2.0);
          cairo->line_to(dirmod * -m_width/2.0, -m_height/2.0);
          cairo->line_to(0.0, 0.0);

          cairo->move_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(dirmod * -m_width/2.0, 0.0);
          cairo->line_to(dirmod * -m_width, m_height/2.0);
          cairo->line_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(dirmod * -m_width/2.0, 0.0);
          break;
        case DOT_AND_TRIANGLE:
          cairo->move_to(dirmod * -m_width/4.0, 0.0);
          cairo->arc(dirmod * -m_width/4.0, 0.0, m_width/4.0, 0.0, 3.0*M_PI);

          cairo->move_to(dirmod * -m_width, -m_height/3.0);
          cairo->line_to(dirmod * -m_width/2.0, 0.0);
          cairo->line_to(dirmod * -m_width, m_height/3.0);
          cairo->line_to(dirmod * -m_width, -m_height/3.0);
          cairo->line_to(dirmod * -m_width/2.0, 0.0);
          break;
        case BOX:
          cairo->rectangle(dirmod * -m_width/2.0, -m_height/4.0, dirmod * m_width/2.0, m_height/2.0);
          cairo->move_to(dirmod * -m_width/4.0, -m_height/2.0);
          cairo->line_to(dirmod * -m_width/4.0, m_height/2.0);
          break;
        case CROW_FOOT:
          cairo->move_to(0.0, -m_height/2.0);
          cairo->line_to(dirmod * -m_width, 0.0);
          cairo->line_to(0.0, m_height/2.0);
          break;
        case CROSS:
          cairo->move_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(dirmod * -m_width, m_height/2.0);
          break;
        case ONE_OR_MANY:
          cairo->move_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(dirmod * -m_width, m_height/2.0);

          cairo->move_to(0.0, -m_height/2.0);
          cairo->line_to(dirmod * -m_width, 0.0);
          cairo->line_to(0.0, m_height/2.0);
          break;
        case ZERO_OR_MANY:
          cairo->save();
          cairo->scale(0.5, 1.0);
          cairo->arc(dirmod * -m_width/4.0*3.0*2.0, 0.0, m_width/2.0, 0.0, 2.0*M_PI);
          cairo->restore();

          cairo->move_to(0.0, -m_height/2.0);
          cairo->line_to(dirmod * -m_width/2.0, 0.0);
          cairo->line_to(0.0, m_height/2.0);
          break;
        case ONE_OR_ZERO:
          cairo->save();
          cairo->scale(0.5, 1.0);
          cairo->arc(dirmod * -m_width/4.0*3.0*2.0, 0.0, m_width/2.0, 0.0, 2.0*M_PI);
          cairo->restore();

          cairo->move_to(dirmod * -m_width/4.0, -m_height/2.0);
          cairo->line_to(dirmod * -m_width/4.0, m_height/2.0);
          break;
        case ONE_EXACTLY:
          cairo->move_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(dirmod * -m_width, m_height/2.0);

          cairo->move_to(dirmod * -m_width/2.0, -m_height/2.0);
          cairo->line_to(dirmod * -m_width/2.0, m_height/2.0);
          break;
        case CONCAVE:
          cairo->move_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(0.0, 0.0);
          cairo->line_to(dirmod * -m_width, m_height/2.0);
          cairo->line_to(dirmod * -m_width/4.0*3.0, 0.0);
          cairo->line_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(0.0, 0.0);
          break;
        case ROUND:
          cairo->arc(dirmod * -m_width/2.0, 0.0, m_height/2.0, -M_PI_2, M_PI_2);
          break;
        case OPEN_ROUND:
          cairo->arc(dirmod * -m_width/2.0, 0.0, m_height/2.0, M_PI_2, -M_PI_2);
          break;
        case BACKSLASH:
          cairo->move_to(dirmod * -m_width, -m_height/2.0);
          cairo->line_to(dirmod * -m_width/2.0, m_height/2.0);
          break;
        }
    }

    void Marker::need_redraw() {
			this->set_extents( Region(-m_width-m_outline.width()/2.0, -m_height/2.0-m_outline.width()/2.0, m_width+m_outline.width(), m_height+m_outline.width()) );
        Drawable::need_redraw();
    }

    sigc::signal<void> Marker::signal_style() {
        return m_signal_style;
    }

    sigc::signal<void> Marker::signal_size() {
        return m_signal_size;
    }

    sigc::signal<void> Marker::signal_facing() {
      return m_signal_facing;
    }


}
