/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "rectangle.h"

namespace Papyrus
{

  Rectangle::Rectangle( double w, double h, Cairo::RefPtr<Cairo::Pattern> fill ) :
      Shape( fill ),
      m_width( w ),
      m_height( h )
  {
    m_centroid_x = m_width / 2.0;
    m_centroid_y = m_height / 2.0;
  }

  Rectangle::Rectangle( double w, double h, Cairo::RefPtr<Cairo::Pattern> fill, const LineStyle& outline ) :
      Shape( fill, outline ),
      m_width( w ),
      m_height( h )
  {
    m_centroid_x = m_width / 2.0;
    m_centroid_y = m_height / 2.0;
  }

  Rectangle::pointer Rectangle::create( double w,
                                        double h,
                                        Cairo::RefPtr<Cairo::Pattern> fill )
  {
    return Rectangle::pointer( new Rectangle( w, h, fill ) );
  }

  Rectangle::pointer Rectangle::create( double w,
                                        double h,
                                        Cairo::RefPtr<Cairo::Pattern> fill,
                                        const LineStyle& outline )
  {
    return Rectangle::pointer( new Rectangle( w, h, fill, outline ) );
  }

  Rectangle::~Rectangle()
  {}

  double Rectangle::width()
  {
    return m_width;
  }

  double Rectangle::height()
  {
    return m_height;
  }

  void Rectangle::width_height( double& w, double& h )
  {
    w = m_width;
    h = m_height;
  }

  void Rectangle::set_width( double w )
  {
    m_width = w;
    m_centroid_x = m_width / 2.0;
    this->shape_changed();
    m_signal_size.emit();
    m_signal_centroid.emit();
  }

  void Rectangle::set_height( double h )
  {
    m_height = h;
    m_centroid_y = m_height / 2.0;
    this->shape_changed();
    m_signal_size.emit();
    m_signal_centroid.emit();
  }

  void Rectangle::set_width_height( double w, double h )
  {
    m_width = w;
    m_centroid_x = m_width / 2.0;
    m_height = h;
    m_centroid_y = m_height / 2.0;
    this->shape_changed();
    m_signal_size.emit();
    m_signal_centroid.emit();
  }

  void Rectangle::set_xywh( double x, double y, double w, double h )
  {
    m_tx = x;
    m_ty = y;
    m_width = w;
    m_centroid_x = m_width / 2.0;
    m_height = h;
    m_centroid_y = m_height / 2.0;
    this->shape_changed();
    m_signal_xy.emit();
    m_signal_size.emit();
    m_signal_centroid.emit();
  }

  void Rectangle::draw_shape( Cairo::RefPtr<Cairo::Context> cairo )
  {
    cairo->rectangle( 0.0, 0.0, m_width, m_height );
  }

  void Rectangle::need_redraw()
  {
    this->set_extents( Region( 0.0, 0.0, m_width, m_height ) );
    Drawable::need_redraw();
  }

  sigc::signal<void> Rectangle::signal_size()
  {
    return m_signal_size;
  }

}
