/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSSELECTOR_H
#define PAPYRUSSELECTOR_H

#include <list>

#include <papyrus/enums.h>
#include <papyrus/smart_pointer.h>
#include <papyrus/controller.h>
#include <papyrus/group.h>

namespace Papyrus
{

  /**
   * This class is in more of a preview form than final form
   *
   * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   */
  class Selector : public Controller
  {
    public:

      typedef PapyrusSmartPointer<Selector> pointer;

      /**
       * typedefs a container to be used to define the selection
       *
       * A list was chosen to allow rapid deletion in the middle of a list, in
       * particular when an item is selected twice and removed from what is likely
       * to be the middle of the list and placed at the end.
       */
      typedef std::list<Drawable::pointer> Selection;

      Selector( Group::pointer select_from_group = Group::pointer(),
                SelectionMode mode = SELECT_PRESS,
                int selection_depth = 1 );

      Selector( Group::pointer select_from_group,
                Group::pointer selection_group,
                SelectionMode mode = SELECT_PRESS,
                int selection_depth = 1,
                bool add_drawable_only = true );

      static Selector::pointer create( Group::pointer select_from_group = Group::pointer(),
                                       SelectionMode mode = SELECT_PRESS,
                                       int selection_depth = 1 );

      static Selector::pointer create( Group::pointer select_from_group,
                                       Group::pointer selection_group,
                                       SelectionMode mode = SELECT_PRESS,
                                       int selection_depth = 1,
                                       bool add_drawable_only = true );

      virtual ~Selector();

      Group::pointer select_from_group();

      void set_select_from_group( Group::pointer select_from_group );

      Group::pointer selection_group();

      virtual void set_selection_group( Group::pointer selection_group, bool add_drawable_only = true );

      void set_select_mode( unsigned mode );

      void enable_select_mode( unsigned mode );

      void disable_select_mode( unsigned mode );

      unsigned select_mode();

      int selection_depth();

      void set_selection_depth( int selection_depth );

      void clear();

      const Selection& selection();

      void add_to_selection( Drawable::pointer drawable );

      void remove_from_selection( Drawable::pointer drawable );

      bool in_selection( Drawable::pointer drawable );

      const Region& extents();

      sigc::signal<void, Drawable::pointer> signal_selected();

      sigc::signal<void, Drawable::pointer> signal_deselected();

      sigc::signal<void> signal_cleared();

    protected:
      unsigned m_select_mode;
      Selection m_selection;
      int m_selection_depth;
      Group::pointer m_select_from_group;
      Group::pointer m_selection_group;
      bool m_add_drawable_only;
      Drawable::pointer m_picked;
      double m_last_position[ 2 ];
      Region m_extents;

      sigc::signal<void, Drawable::pointer> m_signal_selected;

      sigc::signal<void, Drawable::pointer> m_signal_deselected;

      sigc::signal<void> m_signal_cleared;

      virtual bool on_button_press( const Event::ButtonPress& event );
      virtual bool on_button_release( const Event::ButtonRelease& event );
      virtual bool on_motion( const Event::Motion& event );

  };

}

#endif
