/***************************************************************************
*   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef PAPYRUSGTKMARKERCOMBOBOX_H
#define PAPYRUSGTKMARKERCOMBOBOX_H

#include <gtkmm.h>

#include <papyrus.h>

namespace PapyrusGtk {

  extern const char* marker_string[];

  /**
   * A drop-down combobox that is loaded with graphical examples of all marker styles
   * styles available in Papyrus.
   *
   * \image html screenshot_markercombobox_collapsed.png "MarkerComboBox collapsed"
   *
   * \image html screenshot_markercombobox_expanded.png "MarkerComboBox expanded"
   *
   * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   */
class MarkerComboBox : public Gtk::HBox
{
public:
  MarkerComboBox(Papyrus::Marker::FACING facing=Papyrus::Marker::RIGHT, int active = 0);

    ~MarkerComboBox();

    Papyrus::Marker::FACING facing();
    void set_facing(Papyrus::Marker::FACING facing);

		Papyrus::Marker::STYLE style();
		void set_style(Papyrus::Marker::STYLE style);

    Glib::SignalProxy0<void>  signal_changed ();

protected:
  Gtk::ComboBox m_combobox;
  Gtk::ToggleButton m_left, m_right;
  Gtk::Tooltips m_tips;
  Papyrus::Marker::FACING m_facing;

  class ModelColumns: public Gtk::TreeModelColumnRecord {
    public:
      ModelColumns() {
        add( picture );
        add( name );
      }

      Gtk::TreeModelColumn<Glib::RefPtr<Gdk::Pixbuf> > picture;
      Gtk::TreeModelColumn<Glib::ustring> name;
  };

  Glib::RefPtr<Gtk::ListStore> m_store;
  ModelColumns m_columns;
  std::vector<Glib::RefPtr<Gdk::Pixbuf> > m_pixbuf;

  void load_pixbufs();

};

}

#endif
